import django.db.models
import django.contrib.auth.models
import django.template.defaultfilters
import django.utils.timezone
import django.urls


class Geolist(django.db.models.Model):

    title = django.db.models.CharField(max_length=128, unique=True)
    slug = django.db.models.SlugField(null=True, blank=True, unique=True)
    cover = django.db.models.URLField()
    description = django.db.models.TextField(blank=True, null=True)
    background = django.db.models.URLField(blank=True, null=True)
    anchor = django.db.models.TextField(blank=True, null=True)

    class Meta:

        ordering = ["title"]

    def __str__(self):
        return self.title

    def save(self, *args, **kwargs):
        self.slug = django.template.defaultfilters.slugify(self.title)
        return super().save(*args, **kwargs)
    
    def get_absolute_url(self):
        return django.urls.reverse("geolist:list", kwargs={"slug": self.slug})
    

class GeolistItem(django.db.models.Model):

    geolist = django.db.models.ForeignKey(Geolist, on_delete=django.db.models.CASCADE, related_name="items")
    label = django.db.models.CharField(max_length=128)
    latitude = django.db.models.FloatField()
    longitude = django.db.models.FloatField()
    elevation = django.db.models.FloatField(blank=True, null=True)

    class Meta:

        ordering = ["-latitude"]  # North to South

    def __str__(self):
        return self.label

    def visited(self, user):
        query = self.visits.filter(history__user=user)
        if query.exists():
            return True, query.get().date_visited
        return False, None


class History(django.db.models.Model):

    user = django.db.models.ForeignKey(django.contrib.auth.models.User, on_delete=django.db.models.CASCADE, related_name="histories")
    geolist = django.db.models.ForeignKey(Geolist, on_delete=django.db.models.CASCADE, related_name="histories")
    progress = django.db.models.FloatField(default=0)
    date_creation = django.db.models.DateTimeField(blank=True, null=True)

    class Meta:

        ordering = ["-date_creation"]
    
    def __str__(self):
        return "%s - %s" % (self.user, self.geolist)

    def save(self, *args, **kwargs):
        if not self.date_creation:
            self.date_creation = django.utils.timezone.now()
        return super().save(*args, **kwargs)


class Visit(django.db.models.Model):

    history = django.db.models.ForeignKey(History, on_delete=django.db.models.CASCADE, related_name="visits")
    item = django.db.models.ForeignKey(GeolistItem, on_delete=django.db.models.CASCADE, related_name="visits")
    date_creation = django.db.models.DateTimeField(blank=True, null=True)
    date_visited = django.db.models.DateTimeField(blank=True, null=True)

    class Meta:

        ordering = ["-date_creation"]
    
    def __str__(self):
        return "%s - %s" % (self.history.user, self.item.label)

    def save(self, *args, **kwargs):
        if not self.date_creation:
            self.date_creation = django.utils.timezone.now()
        return super().save(*args, **kwargs)
