const NS_SVG = "http://www.w3.org/2000/svg";
var INSTANCE;


function project(latitude, longitude) {
    const w = 679.0;
    const h = 724.0;
    let theta = Math.PI * latitude / 180.0;
    let phi = Math.PI * longitude / 180.0;
    let x = w * (phi + Math.PI) / (2 * Math.PI);
    let y = .5 * h - (w / (2 * Math.PI)) * Math.log(Math.tan(.25 * Math.PI + .5 * theta));
    return {
        "x": x,
        "y": y
    }
}


function createScaler(points) {
    let minX, maxX, minY, maxY;
    points.forEach(point => {
        let pos = project(point.latitude, point.longitude);
        if (minX == null | minX > pos.x) minX = pos.x;
        if (maxX == null | maxX < pos.x) maxX = pos.x;
        if (minY == null | minY > pos.y) minY = pos.y;
        if (maxY == null | maxY < pos.y) maxY = pos.y;
    });
    let transformer = (pos) => {
        return {
            "x": 1000 * (pos.x - minX),
            "y": 1000 * (pos.y - minY)
        }
    }
    return transformer;
}

function getViewBox(points, scaler) {
    let minX, maxX, minY, maxY;
    points.forEach(point => {
        let pos = scaler(project(point.latitude, point.longitude));
        if (minX == null | minX > pos.x) minX = pos.x;
        if (maxX == null | maxX < pos.x) maxX = pos.x;
        if (minY == null | minY > pos.y) minY = pos.y;
        if (maxY == null | maxY < pos.y) maxY = pos.y;
    });
    let width = maxX - minX;
    let height = maxY - minY;
    let paddingX = 0.05 * width;
    let paddingY = 0.05 * height;
    console.log(`${minX - paddingX} ${minY - paddingY} ${width + 2 * paddingX} ${height + 2 * paddingY}`);
    return `${minX - paddingX} ${minY - paddingY} ${width + 2 * paddingX} ${height + 2 * paddingY}`;
}


function inflatePoints(points) {
    let scene = document.getElementById("svgscene");
    scene.innerHTML = "<g id='background'></g>";
    let group = document.createElementNS(NS_SVG, "g");
    group.setAttribute("id", "foreground");
    let scaler = createScaler(points);
    points.forEach(point => {
        let item = document.createElementNS(NS_SVG, "circle");
        item.classList.add("geolist-item");
        if (point.visited) item.classList.add("visited");
        let pos = scaler(project(point.latitude, point.longitude));
        item.setAttribute("cx", pos.x);
        item.setAttribute("cy", pos.y);
        item.setAttribute("r", 2);
        group.appendChild(item);
        let tooltip = document.getElementById("tooltip");

        function showTooltip() {
            tooltip.querySelector(".card-title").innerHTML = point.label;
            if (point.elevation != null) {
                tooltip.querySelector(".card-subtitle").innerHTML = point.elevation + " m";
            }
            tooltip.classList.remove("hidden");
            let rect = item.getBoundingClientRect();
            tooltip.style.top = (rect.top + 0.5 * rect.height) + "px";
            tooltip.style.left = (rect.right + 3) + "px";
        }

        function hideTooltip() {
            tooltip.classList.add("hidden");
        }

        item.addEventListener("mouseenter", showTooltip);
        item.addEventListener("mouseleave", hideTooltip);

        let onClickHandler = function() {
            let modal = document.getElementById("modal-visit");
            modal.querySelector(".modal-title").innerHTML = point.label;
            modal.querySelector(".modal-subtitle").innerHTML = point.latitude + ", " + point.longitude;
            modal.querySelector("input[name='visited']").checked = point.visited;
            if (point.date != null) {
                modal.querySelector("input[name='date']").value = point.date;
            } else {
                modal.querySelector("input[name='date']").value = "";
            }
            if (point.visited) {
                modal.querySelector("#input-date").classList.remove("disabled");
            } else {
                modal.querySelector("#input-date").classList.add("disabled");
            }
            modal.querySelector("input[name='itemid']").value = point.id;
            INSTANCE.pause();
            modal.classList.add("active");
        }

        item.addEventListener("click", onClickHandler);
        item.addEventListener("touchend", () => {
            console.log("Touch click");
            onClickHandler();
        });

    });
    scene.appendChild(group);
    scene.setAttribute("viewBox", getViewBox(points, scaler));

    INSTANCE = panzoom(
        scene, {
            zoomDoubleClickSpeed: 1,
        });
    // instance.setTransformOrigin(null);


    scene.addEventListener("mousedown", function(e) {
        scene.classList.add("grabbing");
    });
    scene.addEventListener("mouseup", function(e) {
        scene.classList.remove("grabbing");
    });
    scene.addEventListener("touchstart", function(e) {
        scene.classList.add("grabbing");
    });
    scene.addEventListener("touchend", function(e) {
        scene.classList.remove("grabbing");
    });

    if (URL_BACKGROUND != null) {
        fetch(URL_BACKGROUND).then(res => res.text()).then(text => {
            let parser = new DOMParser();
            let doc = parser.parseFromString(text, "image/svg+xml");
            document.getElementById("background").appendChild(doc.documentElement);
            if (ANCHOR) {
                let pos1 = scaler(project(ANCHOR.lat1, ANCHOR.lng1));
                let pos2 = scaler(project(ANCHOR.lat2, ANCHOR.lng2));
                let scale = (pos1.y - pos2.y) / (ANCHOR.y1 - ANCHOR.y2);
                let translateX = pos1.x - scale * ANCHOR.x1;
                let translateY = pos1.y - scale * ANCHOR.y1;
                document.getElementById("background").style.transform = `translate(${translateX}px, ${translateY}px) scale(${scale})`;
            }
        });
    }

}

window.addEventListener("load", () => {
    fetch(URL_POINTS).then(res => res.json()).then(data => { inflatePoints(data.points) });
    let input = document.querySelector("input[name='visited']");
    input.addEventListener("change", (event) => {
        if (input.checked) {
            document.getElementById("input-date").classList.remove("disabled");
        } else {
            document.getElementById("input-date").classList.add("disabled");
        }
    });
});